@@ -62,7 +62,7 @@
    . .   - sane_get_parameters() : returns actual scan-parameters
    . .   - sane_read() : read image-data (from pipe)
    . . - sane_cancel() : cancel operation, kill reader_process
-   
+
    . - sane_close() : close opened scanner-device, do_cancel, free buffer and handle
    - sane_exit() : terminate use of backend, free devicename and device-struture
 */
@@ -140,7 +140,7 @@ static const SANE_String_Const mode_list[] = {
   SANE_VALUE_SCAN_MODE_HALFTONE,
   SANE_VALUE_SCAN_MODE_GRAY,
   SANE_VALUE_SCAN_MODE_COLOR,
-  0     
+  0
 };
 
 /* modification for FS2710 */
@@ -193,11 +193,6 @@ static const SANE_String_Const tpu_filmtype_list[] = {
   0
 };
 
-static const SANE_String_Const papersize_list[] = {
-  "A4", "Letter", "B5", "Maximal",
-  0
-};
-
 /**************************************************/
 
 static const SANE_Range u8_range = {
@@ -325,7 +320,6 @@ static SANE_Status
 sense_handler (int scsi_fd, u_char * result, void *arg)
 {
   static char me[] = "canon_sense_handler";
-  CANON_Device *dev = (CANON_Device *) arg;
   u_char sense;
   int asc;
   char *sense_str = NULL;
@@ -342,210 +336,198 @@ sense_handler (int scsi_fd, u_char * result, void *arg)
 
   status = SANE_STATUS_GOOD;
 
-  /* If the sense handler is invoked before the scanner has been
-     identified, pretend that nothing has happened. */
-  if (strncmp(dev->sane.vendor, "CANON", 5) != 0) return (status);
-
-  if (dev && dev->info.is_scsi2)
+  DBG(11, "sense data interpretation for SCSI-2 devices\n");
+  sense = result[2] & 0x0f;		/* extract the sense key */
+  if (result[7] > 3)		/* additional sense code available? */
     {
-      DBG(11, "sense data interpretation for SCSI-2 devices\n");
-      sense = result[2] & 0x0f;		/* extract the sense key */
-      if (result[7] > 3)		/* additional sense code available? */
-	{
-	  asc = (result[12] << 8) + result[13];	/* 12: additional sense code */
-	}					/* 13: a.s.c. qualifier */
-      else
-	asc = 0xffff;
-
-      switch (sense)
-	{
-	case 0x00:
-	  DBG(11, "sense category: no error\n");
-	  status = SANE_STATUS_GOOD;
-	  break;
-
-	case 0x01:
-	  DBG(11, "sense category: recovered error\n");
-	  switch (asc)
-	    {
-	    case 0x3700:
-	      sense_str = SANE_I18N("rounded parameter");
-	      break;
-	    default:
-	      sense_str = SANE_I18N("unknown");
-	    }
-	  status = SANE_STATUS_GOOD;
-	  break;
-
-	case 0x03:
-	  DBG(11, "sense category: medium error\n");
-	  switch (asc)
-	    {
-	    case 0x8000:
-	      sense_str = SANE_I18N("ADF jam");
-	      break;
-	    case 0x8001:
-	      sense_str = SANE_I18N("ADF cover open");
-	      break;
-	    default:
-	      sense_str = SANE_I18N("unknown");
-	    }
-	  status = SANE_STATUS_IO_ERROR;
-	  break;
-
-	case 0x04:
-	  DBG(11, "sense category: hardware error\n");
-	  switch (asc)
-	    {
-	    case 0x6000:
-	      sense_str = SANE_I18N("lamp failure");
-	      break;
-	    case 0x6200:
-	      sense_str = SANE_I18N("scan head positioning error");
-	      break;
-	    case 0x8001:
-	      sense_str = SANE_I18N("CPU check error");
-	      break;
-	    case 0x8002:
-	      sense_str = SANE_I18N("RAM check error");
-	      break;
-	    case 0x8003:
-	      sense_str = SANE_I18N("ROM check error");
-	      break;
-	    case 0x8004:
-	      sense_str = SANE_I18N("hardware check error");
-	      break;
-	    case 0x8005:
-	      sense_str = SANE_I18N("transparency unit lamp failure");
-	      break;
-	    case 0x8006:
-	      sense_str = SANE_I18N("transparency unit scan head "
-	      "positioning failure");
-	      break;
-	    default:
-	      sense_str = SANE_I18N("unknown");
-	    }
-	  status = SANE_STATUS_IO_ERROR;
-	  break;
-
-	case 0x05:
-	  DBG(11, "sense category: illegal request\n");
-	  switch (asc)
-	    {
-	    case 0x1a00:
-	      sense_str = SANE_I18N("parameter list length error");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    case 0x2000:
-	      sense_str = SANE_I18N("invalid command operation code");
-	      status = SANE_STATUS_UNSUPPORTED;
-	      break;
-	    case 0x2400:
-	      sense_str = SANE_I18N("invalid field in CDB");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    case 0x2500:
-	      sense_str = SANE_I18N("unsupported LUN");
-	      status = SANE_STATUS_UNSUPPORTED;
-	      break;
-	    case 0x2600:
-	      sense_str = SANE_I18N("invalid field in parameter list");
-	      status = SANE_STATUS_UNSUPPORTED;
-	      break;
-	    case 0x2c00:
-	      sense_str = SANE_I18N("command sequence error");
-	      status = SANE_STATUS_UNSUPPORTED;
-	      break;
-	    case 0x2c01:
-	      sense_str = SANE_I18N("too many windows specified");
-	      status = SANE_STATUS_UNSUPPORTED;
-	      break;
-	    case 0x3a00:
-	      sense_str = SANE_I18N("medium not present");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    case 0x3d00:
-	      sense_str = SANE_I18N("invalid bit IDENTIFY message");
-	      status = SANE_STATUS_UNSUPPORTED;
-	      break;
-	    case 0x8002:
-	      sense_str = SANE_I18N("option not connect");
-	      status = SANE_STATUS_UNSUPPORTED;
-	      break;
-	    default:
-	      sense_str = SANE_I18N("unknown");
-	      status = SANE_STATUS_UNSUPPORTED;
-	    }
-	  break;
-
-	case 0x06:
-	  DBG(11, "sense category: unit attention\n");
-	  switch (asc)
-	    {
-	    case 0x2900:
-	      sense_str = SANE_I18N("power on reset / bus device reset");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    case 0x2a00:
-	      sense_str = SANE_I18N("parameter changed by another initiator");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    default:
-	      sense_str = SANE_I18N("unknown");
-	      status = SANE_STATUS_IO_ERROR;
-	    }
-	  break;
-
-	case 0x0b:
-	  DBG(11, "sense category: non-standard\n");
-	  switch (asc)
-	    {
-	    case 0x0000:
-	      sense_str = SANE_I18N("no additional sense information");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    case 0x4500:
-	      sense_str = SANE_I18N("reselect failure");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    case 0x4700:
-	      sense_str = SANE_I18N("SCSI parity error");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    case 0x4800:
-	      sense_str = SANE_I18N("initiator detected error message "
-	      "received");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    case 0x4900:
-	      sense_str = SANE_I18N("invalid message error");
-	      status = SANE_STATUS_UNSUPPORTED;
-	      break;
-	    case 0x8000:
-	      sense_str = SANE_I18N("timeout error");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    case 0x8001:
-	      sense_str = SANE_I18N("transparency unit shading error");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    case 0x8003:
-	      sense_str = SANE_I18N("lamp not stabilized");
-	      status = SANE_STATUS_IO_ERROR;
-	      break;
-	    default:
-	      sense_str = SANE_I18N("unknown");
-	      status = SANE_STATUS_IO_ERROR;
-	    }
-	  break;
-	default:
-	  DBG(11, "sense category: else\n");
-	}
-    }
+      asc = (result[12] << 8) + result[13];	/* 12: additional sense code */
+    }					/* 13: a.s.c. qualifier */
   else
+    asc = 0xffff;
+
+  switch (sense)
     {
-      sense_str = SANE_I18N("problem not analyzed (unknown SCSI class)");
+    case 0x00:
+      DBG(11, "sense category: no error\n");
+      status = SANE_STATUS_GOOD;
+      break;
+
+    case 0x01:
+      DBG(11, "sense category: recovered error\n");
+      switch (asc)
+        {
+        case 0x3700:
+          sense_str = SANE_I18N("rounded parameter");
+          break;
+        default:
+          sense_str = SANE_I18N("unknown");
+        }
+      status = SANE_STATUS_GOOD;
+      break;
+
+    case 0x03:
+      DBG(11, "sense category: medium error\n");
+      switch (asc)
+        {
+        case 0x8000:
+          sense_str = SANE_I18N("ADF jam");
+          break;
+        case 0x8001:
+          sense_str = SANE_I18N("ADF cover open");
+          break;
+        default:
+          sense_str = SANE_I18N("unknown");
+        }
       status = SANE_STATUS_IO_ERROR;
+      break;
+
+    case 0x04:
+      DBG(11, "sense category: hardware error\n");
+      switch (asc)
+        {
+        case 0x6000:
+          sense_str = SANE_I18N("lamp failure");
+          break;
+        case 0x6200:
+          sense_str = SANE_I18N("scan head positioning error");
+          break;
+        case 0x8001:
+          sense_str = SANE_I18N("CPU check error");
+          break;
+        case 0x8002:
+          sense_str = SANE_I18N("RAM check error");
+          break;
+        case 0x8003:
+          sense_str = SANE_I18N("ROM check error");
+          break;
+        case 0x8004:
+          sense_str = SANE_I18N("hardware check error");
+          break;
+        case 0x8005:
+          sense_str = SANE_I18N("transparency unit lamp failure");
+          break;
+        case 0x8006:
+          sense_str = SANE_I18N("transparency unit scan head "
+          "positioning failure");
+          break;
+        default:
+          sense_str = SANE_I18N("unknown");
+        }
+      status = SANE_STATUS_IO_ERROR;
+      break;
+
+    case 0x05:
+      DBG(11, "sense category: illegal request\n");
+      switch (asc)
+        {
+        case 0x1a00:
+          sense_str = SANE_I18N("parameter list length error");
+          status = SANE_STATUS_IO_ERROR;
+          break;
+        case 0x2000:
+          sense_str = SANE_I18N("invalid command operation code");
+          status = SANE_STATUS_UNSUPPORTED;
+          break;
+        case 0x2400:
+          sense_str = SANE_I18N("invalid field in CDB");
+          status = SANE_STATUS_IO_ERROR;
+          break;
+        case 0x2500:
+          sense_str = SANE_I18N("unsupported LUN");
+          status = SANE_STATUS_UNSUPPORTED;
+          break;
+        case 0x2600:
+          sense_str = SANE_I18N("invalid field in parameter list");
+          status = SANE_STATUS_UNSUPPORTED;
+          break;
+        case 0x2c00:
+          sense_str = SANE_I18N("command sequence error");
+          status = SANE_STATUS_UNSUPPORTED;
+          break;
+        case 0x2c01:
+          sense_str = SANE_I18N("too many windows specified");
+          status = SANE_STATUS_UNSUPPORTED;
+          break;
+        case 0x3a00:
+          sense_str = SANE_I18N("medium not present");
+          status = SANE_STATUS_IO_ERROR;
+          break;
+        case 0x3d00:
+          sense_str = SANE_I18N("invalid bit IDENTIFY message");
+          status = SANE_STATUS_UNSUPPORTED;
+          break;
+        case 0x8002:
+          sense_str = SANE_I18N("option not connect");
+          status = SANE_STATUS_UNSUPPORTED;
+          break;
+        default:
+          sense_str = SANE_I18N("unknown");
+          status = SANE_STATUS_UNSUPPORTED;
+        }
+      break;
+
+    case 0x06:
+      DBG(11, "sense category: unit attention\n");
+      switch (asc)
+        {
+        case 0x2900:
+          sense_str = SANE_I18N("power on reset / bus device reset");
+          status = SANE_STATUS_GOOD;
+          break;
+        case 0x2a00:
+          sense_str = SANE_I18N("parameter changed by another initiator");
+          status = SANE_STATUS_IO_ERROR;
+          break;
+        default:
+          sense_str = SANE_I18N("unknown");
+          status = SANE_STATUS_IO_ERROR;
+        }
+      break;
+
+    case 0x0b:
+      DBG(11, "sense category: non-standard\n");
+      switch (asc)
+        {
+        case 0x0000:
+          sense_str = SANE_I18N("no additional sense information");
+          status = SANE_STATUS_IO_ERROR;
+          break;
+        case 0x4500:
+          sense_str = SANE_I18N("reselect failure");
+          status = SANE_STATUS_IO_ERROR;
+          break;
+        case 0x4700:
+          sense_str = SANE_I18N("SCSI parity error");
+          status = SANE_STATUS_IO_ERROR;
+          break;
+        case 0x4800:
+          sense_str = SANE_I18N("initiator detected error message "
+          "received");
+          status = SANE_STATUS_IO_ERROR;
+          break;
+        case 0x4900:
+          sense_str = SANE_I18N("invalid message error");
+          status = SANE_STATUS_UNSUPPORTED;
+          break;
+        case 0x8000:
+          sense_str = SANE_I18N("timeout error");
+          status = SANE_STATUS_IO_ERROR;
+          break;
+        case 0x8001:
+          sense_str = SANE_I18N("transparency unit shading error");
+          status = SANE_STATUS_IO_ERROR;
+          break;
+        case 0x8003:
+          sense_str = SANE_I18N("lamp not stabilized");
+          status = SANE_STATUS_IO_ERROR;
+          break;
+        default:
+          sense_str = SANE_I18N("unknown");
+          status = SANE_STATUS_IO_ERROR;
+        }
+      break;
+    default:
+      DBG(11, "sense category: else\n");
     }
   DBG (11, "sense message: %s\n", sense_str);
 #if 0					/* superfluous? [U.D.] */
@@ -853,9 +835,9 @@ attach (const char *devnam, CANON_Device ** devp)
      - whether it has got focus control
      - whether it can optimize image parameters (autoexposure)
      - whether it can calibrate itself
+     - whether it can diagnose itself
      - whether it can eject the media
      - whether it can mirror the scanned data
-     - whether it is an SCSI-2 device (gives status information by "sense key")
      - whether it is a film scanner (or can be used as one)
      - whether it has fixed, hardware-set scan resolutions only
   */
@@ -868,9 +850,9 @@ attach (const char *devnam, CANON_Device ** devp)
       dev->info.can_focus = SANE_TRUE;
       dev->info.can_autoexpose = SANE_TRUE;
       dev->info.can_calibrate = SANE_FALSE;
+      dev->info.can_diagnose = SANE_FALSE;
       dev->info.can_eject = SANE_TRUE;
       dev->info.can_mirror = SANE_TRUE;
-      dev->info.is_scsi2 = SANE_TRUE;
       dev->info.is_filmscanner = SANE_TRUE;
       dev->info.has_fixed_resolutions = SANE_TRUE;
     }
@@ -883,9 +865,9 @@ attach (const char *devnam, CANON_Device ** devp)
       dev->info.can_focus = SANE_TRUE;
       dev->info.can_autoexpose = SANE_TRUE;
       dev->info.can_calibrate = SANE_FALSE;
+      dev->info.can_diagnose = SANE_FALSE;
       dev->info.can_eject = SANE_TRUE;
       dev->info.can_mirror = SANE_TRUE;
-      dev->info.is_scsi2 = SANE_TRUE;
       dev->info.is_filmscanner = SANE_TRUE;
       dev->info.has_fixed_resolutions = SANE_TRUE;
     }
@@ -898,9 +880,9 @@ attach (const char *devnam, CANON_Device ** devp)
       dev->info.can_focus = SANE_FALSE;
       dev->info.can_autoexpose = SANE_FALSE;
       dev->info.can_calibrate = SANE_TRUE;
+      dev->info.can_diagnose = SANE_TRUE;
       dev->info.can_eject = SANE_FALSE;
       dev->info.can_mirror = SANE_FALSE;
-      dev->info.is_scsi2 = SANE_TRUE;
       dev->info.is_filmscanner = SANE_FALSE;
       dev->info.has_fixed_resolutions = SANE_TRUE;
     }
@@ -913,12 +895,27 @@ attach (const char *devnam, CANON_Device ** devp)
       dev->info.can_focus = SANE_FALSE;
       dev->info.can_autoexpose = SANE_FALSE;
       dev->info.can_calibrate = SANE_FALSE;
+      dev->info.can_diagnose = SANE_FALSE;
       dev->info.can_eject = SANE_FALSE;
       dev->info.can_mirror = SANE_FALSE;
-      dev->info.is_scsi2 = SANE_TRUE;
       dev->info.is_filmscanner = SANE_FALSE;
       dev->info.has_fixed_resolutions = SANE_TRUE;
     }
+  else if (!strncmp (str, "IX-4015", 7))	/* IX-4015 */
+    {
+      dev->info.model = IX4015;
+      dev->sane.type = SANE_I18N("flatbed scanner");
+      dev->adf.Status = ADF_STAT_INACTIVE;
+      dev->tpu.Status = TPU_STAT_INACTIVE;
+      dev->info.can_focus = SANE_FALSE;
+      dev->info.can_autoexpose = SANE_TRUE;
+      dev->info.can_calibrate = SANE_FALSE;
+      dev->info.can_diagnose = SANE_TRUE;
+      dev->info.can_eject = SANE_FALSE;
+      dev->info.can_mirror = SANE_TRUE;
+      dev->info.is_filmscanner = SANE_FALSE;
+      dev->info.has_fixed_resolutions = SANE_FALSE;
+    }
   else						/* CS300, CS600 */
     {
       dev->info.model = CS3_600;
@@ -928,9 +925,9 @@ attach (const char *devnam, CANON_Device ** devp)
       dev->info.can_focus = SANE_FALSE;
       dev->info.can_autoexpose = SANE_FALSE;
       dev->info.can_calibrate = SANE_FALSE;
+      dev->info.can_diagnose = SANE_FALSE;
       dev->info.can_eject = SANE_FALSE;
       dev->info.can_mirror = SANE_TRUE;
-      dev->info.is_scsi2 = SANE_TRUE;
       dev->info.is_filmscanner = SANE_FALSE;
       dev->info.has_fixed_resolutions = SANE_FALSE;
     }
@@ -1419,8 +1416,8 @@ init_options (CANON_Scanner * s)
   s->opt[OPT_BIND_HILO].title = SANE_TITLE_RGB_BIND;
   s->opt[OPT_BIND_HILO].desc = SANE_DESC_RGB_BIND;
   s->opt[OPT_BIND_HILO].type = SANE_TYPE_BOOL;
-  s->opt[OPT_BIND_HILO].cap |=
-    (s->hw->info.model == FB620) ? SANE_CAP_INACTIVE : 0;
+  s->opt[OPT_BIND_HILO].cap |= (s->hw->info.model == FB620 ||
+    s->hw->info.model == IX4015) ? SANE_CAP_INACTIVE : 0;
   s->val[OPT_BIND_HILO].w = SANE_TRUE;
 
   /* highlight point for red   */
@@ -1453,6 +1450,8 @@ init_options (CANON_Scanner * s)
   s->opt[OPT_HILITE_G].unit = SANE_UNIT_NONE;
   s->opt[OPT_HILITE_G].constraint_type = SANE_CONSTRAINT_RANGE;
   s->opt[OPT_HILITE_G].constraint.range = &s->hw->info.HiliteG_range;
+  s->opt[OPT_HILITE_G].cap |=
+    (s->hw->info.model == IX4015) ? SANE_CAP_INACTIVE : 0;
   s->val[OPT_HILITE_G].w = 255;
 
   /* shadow point for green */
@@ -1463,6 +1462,8 @@ init_options (CANON_Scanner * s)
   s->opt[OPT_SHADOW_G].unit = SANE_UNIT_NONE;
   s->opt[OPT_SHADOW_G].constraint_type = SANE_CONSTRAINT_RANGE;
   s->opt[OPT_SHADOW_G].constraint.range = &s->hw->info.ShadowG_range;
+  s->opt[OPT_SHADOW_G].cap |=
+    (s->hw->info.model == IX4015) ? SANE_CAP_INACTIVE : 0;
   s->val[OPT_SHADOW_G].w = 0;
 
   /* highlight point for blue  */
@@ -1610,8 +1611,8 @@ init_options (CANON_Scanner * s)
   s->opt[OPT_CALIBRATION_GROUP].title = SANE_I18N("Calibration");
   s->opt[OPT_CALIBRATION_GROUP].desc = "";
   s->opt[OPT_CALIBRATION_GROUP].type = SANE_TYPE_GROUP;
-  s->opt[OPT_CALIBRATION_GROUP].cap |=
-    (s->hw->info.can_calibrate) ? 0 : SANE_CAP_INACTIVE;
+  s->opt[OPT_CALIBRATION_GROUP].cap |= (s->hw->info.can_calibrate ||
+    s->hw->info.can_diagnose) ? 0 : SANE_CAP_INACTIVE;
   s->opt[OPT_CALIBRATION_GROUP].constraint_type = SANE_CONSTRAINT_NONE;
 
   /* calibration now */
@@ -1633,7 +1634,7 @@ init_options (CANON_Scanner * s)
   s->opt[OPT_SCANNER_SELF_DIAGNOSTIC].type = SANE_TYPE_BUTTON;
   s->opt[OPT_SCANNER_SELF_DIAGNOSTIC].unit = SANE_UNIT_NONE;
   s->opt[OPT_SCANNER_SELF_DIAGNOSTIC].cap |=
-    (s->hw->info.can_calibrate) ? 0 : SANE_CAP_INACTIVE;
+    (s->hw->info.can_diagnose) ? 0 : SANE_CAP_INACTIVE;
   s->opt[OPT_SCANNER_SELF_DIAGNOSTIC].constraint_type = SANE_CONSTRAINT_NONE;
   s->opt[OPT_SCANNER_SELF_DIAGNOSTIC].constraint.range = NULL;
 
@@ -1665,7 +1666,9 @@ init_options (CANON_Scanner * s)
   s->opt[OPT_EJECT_AFTERSCAN].cap |=
     (s->hw->info.can_eject) ? 0 : SANE_CAP_INACTIVE;
   s->opt[OPT_EJECT_AFTERSCAN].type = SANE_TYPE_BOOL;
-  s->val[OPT_EJECT_AFTERSCAN].w = SANE_FALSE;
+  /* IX-4015 requires medium_position command after cancel */
+  s->val[OPT_EJECT_AFTERSCAN].w =
+    (s->hw->info.model == IX4015) ? SANE_TRUE : SANE_FALSE;
 
   /* eject before exit */
   s->opt[OPT_EJECT_BEFOREEXIT].name = "eject-before-exit";
