@@ -42,7 +42,7 @@
    allows managing an arbitrary number of SANE backends by using
    dynamic linking to load backends on demand.  */
 
-/* Please increase version number with every change 
+/* Please increase version number with every change
    (don't forget to update dll.desc) */
 #define DLL_VERSION "1.0.13"
 
@@ -69,6 +69,26 @@
 #if defined(HAVE_DLOPEN) && defined(HAVE_DLFCN_H)
 # include <dlfcn.h>
 
+  /* This works around a pedantic GCC compiler warning.  The ISO C
+     standard says that the behaviour of converting an object pointer
+     like the void * returned by dlsym() to a function pointer like
+     void *(*)() is implementation defined.  POSIX though guarantees
+     that this works fine.
+
+     Workaround based on http://stackoverflow.com/a/36385690.  Turns
+     off pedantic warnings for the duration of the definition only.
+   */
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wpedantic"
+typedef void *(*func_ptr)(void);
+
+func_ptr
+posix_dlsym (void *handle, const char *func)
+{
+  return dlsym (handle, func);
+}
+# pragma GCC diagnostic pop
+
   /* Older versions of dlopen() don't define RTLD_NOW and RTLD_LAZY.
      They all seem to use a mode of 1 to indicate RTLD_NOW and some do
      not support RTLD_LAZY at all.  Hence, unless defined, we define
@@ -333,7 +353,7 @@ load (struct backend *be)
 	image_id id = -1;
 	int i, w;
 	directory_which which[3] = { B_USER_ADDONS_DIRECTORY, B_COMMON_ADDONS_DIRECTORY, B_BEOS_ADDONS_DIRECTORY };
-	
+
 	/* look for config files in SANE/conf */
 	for (w = 0; (w < 3) && (id < 0) && (find_directory(which[w],0,true,path,PATH_MAX) == 0); w++)
 	{
@@ -353,7 +373,7 @@ load (struct backend *be)
 			continue; /* try next path */
 		}
     	be->handle=(void *)id;
-	    	
+
 		for (i = 0; i < NUM_OPS; ++i)
     	{
       		void *(*op) ();
@@ -430,15 +450,17 @@ load (struct backend *be)
 
   if (path)
     {
-      src_len = strlen (path) + strlen (LIBDIR) + 1 + 1;
+      src_len = strlen (path) + strlen (DIR_SEP) + strlen(LIBDIR) + 1;
       src = malloc (src_len);
       if (!src)
 	{
 	  DBG (1, "load: malloc failed: %s\n", strerror (errno));
 	  return SANE_STATUS_NO_MEM;
 	}
+      if (orig_src)
+        free (orig_src);
       orig_src = src;
-      snprintf (src, src_len, "%s:%s", path, LIBDIR);
+      snprintf (src, src_len, "%s%s%s", path, DIR_SEP, LIBDIR);
     }
   else
     {
@@ -470,7 +492,7 @@ load (struct backend *be)
 	break;
       DBG (4, "load: couldn't open `%s' (%s)\n", libname, strerror (errno));
 
-#ifdef ALT_POSTFIX      
+#ifdef ALT_POSTFIX
       /* Some platforms have two ways of storing their libraries, try both
 	 postfixes */
       snprintf (libname, sizeof (libname), "%s/" PREFIX "%s" ALT_POSTFIX,
@@ -534,7 +556,7 @@ load (struct backend *be)
 
       /* First try looking up the symbol without a leading underscore. */
 #ifdef HAVE_DLOPEN
-      op = (void *(*)(void)) dlsym (be->handle, funcname + 1);
+      op = posix_dlsym (be->handle, funcname + 1);
 #elif defined(HAVE_SHL_LOAD)
       shl_findsym ((shl_t *) & (be->handle), funcname + 1, TYPE_UNDEFINED,
 		   &op);
@@ -559,7 +581,7 @@ load (struct backend *be)
 	{
 	  /* Try again, with an underscore prepended. */
 #ifdef HAVE_DLOPEN
-	  op = (void *(*)(void)) dlsym (be->handle, funcname);
+	  op = posix_dlsym (be->handle, funcname);
 #elif defined(HAVE_SHL_LOAD)
 	  shl_findsym (be->handle, funcname, TYPE_UNDEFINED, &op);
 #elif defined(HAVE_NSLINKMODULE)
@@ -860,7 +882,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   char path[1024];
   directory_which which[3] = { B_USER_ADDONS_DIRECTORY, B_COMMON_ADDONS_DIRECTORY, B_BEOS_ADDONS_DIRECTORY };
   int i;
-#endif	
+#endif
 
   DBG_INIT ();
 
@@ -912,7 +934,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
     }
   fclose (fp);
 
-#else  
+#else
 	/* no ugly config files, just get scanners from their ~/config/add-ons/SANE */
 	/* look for drivers */
 	for (i = 0; i < 3; i++)
@@ -921,13 +943,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 			continue;
 		strcat(path,"/SANE/");
 		dir=opendir(path);
-		if(!dir) continue; 
+		if(!dir) continue;
 
 		while((dirent=readdir(dir)))
 		{
 			if((strcmp(dirent->d_name,".")==0) || (strcmp(dirent->d_name,"..")==0)) continue;
 			if((strcmp(dirent->d_name,"dll")==0)) continue;
-			add_backend(dirent->d_name,0); 
+			add_backend(dirent->d_name,0);
 		}
 		closedir(dir);
 	}
@@ -1133,7 +1155,8 @@ sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
 SANE_Status
 sane_open (SANE_String_Const full_name, SANE_Handle * meta_handle)
 {
-  const char *be_name, *dev_name;
+  char *be_name;
+  const char *dev_name;
   struct meta_scanner *s;
   SANE_Handle handle;
   struct backend *be;
@@ -1156,16 +1179,7 @@ sane_open (SANE_String_Const full_name, SANE_Handle * meta_handle)
   dev_name = strchr (full_name, ':');
   if (dev_name)
     {
-#ifdef strndupa
-      be_name = strndupa (full_name, dev_name - full_name);
-#else
-      char *tmp;
-
-      tmp = alloca (dev_name - full_name + 1);
-      memcpy (tmp, full_name, dev_name - full_name);
-      tmp[dev_name - full_name] = '\0';
-      be_name = tmp;
-#endif
+      be_name = strndup(full_name, dev_name - full_name);
       ++dev_name;		/* skip colon */
     }
   else
@@ -1173,10 +1187,13 @@ sane_open (SANE_String_Const full_name, SANE_Handle * meta_handle)
       /* if no colon interpret full_name as the backend name; an empty
          backend device name will cause us to open the first device of
          that backend.  */
-      be_name = full_name;
+      be_name = strdup(full_name);
       dev_name = "";
     }
 
+  if (!be_name)
+    return SANE_STATUS_NO_MEM;
+
   if (!be_name[0])
     be = first_backend;
   else
@@ -1188,8 +1205,12 @@ sane_open (SANE_String_Const full_name, SANE_Handle * meta_handle)
     {
       status = add_backend (be_name, &be);
       if (status != SANE_STATUS_GOOD)
-	return status;
+        {
+          free(be_name);
+          return status;
+        }
     }
+  free(be_name);
 
   if (!be->inited)
     {
