@@ -29,6 +29,7 @@
  *	20	usb cmd counters
  *	18	sane_read
  *	17	setvalue, getvalue, control_option
+ *	16	gamma table
  *	15	e2_send, e2_recv calls
  *	13	e2_cmd_info_block
  *	12	epson_cmd_simple
@@ -240,7 +241,7 @@ enum {
  * therefore I define two different sets of arrays, plus one set of
  * variables that get set to the actally used params and list arrays at runtime.
  */
- 
+
 static int gamma_params_ab[] = {
 	0x01,
 	0x03,
@@ -302,7 +303,6 @@ static const SANE_String_Const bay_list[] = {
 
 /* minimum, maximum, quantization */
 static const SANE_Range u8_range = { 0, 255, 0 };
-static const SANE_Range s8_range = { -127, 127, 0 };
 static const SANE_Range fx_range = { SANE_FIX(-2.0), SANE_FIX(2.0), 0 };
 
 static const SANE_Range outline_emphasis_range = { -2, 2, 0 };
@@ -355,10 +355,12 @@ print_params(const SANE_Parameters params)
 static void
 close_scanner(Epson_Scanner *s)
 {
+	int i;
+
 	DBG(7, "%s: fd = %d\n", __func__, s->fd);
 
 	if (s->fd == -1)
-		return;
+		goto free;
 
 	/* send a request_status. This toggles w_cmd_count and r_cmd_count */
 	if (r_cmd_count % 2)
@@ -380,6 +382,14 @@ close_scanner(Epson_Scanner *s)
 	}
 
 	s->fd = -1;
+
+free:
+	for (i = 0; i < LINES_SHUFFLE_MAX; i++) {
+		if (s->line_buffer[i] != NULL)
+			free(s->line_buffer[i]);
+	}
+
+	free(s);
 }
 
 static void
@@ -409,8 +419,8 @@ e2_network_discovery(void)
 	FD_SET(fd, &rfds);
 
 	sanei_udp_set_nonblock(fd, SANE_TRUE);
-	if (select(fd + 1, &rfds, NULL, NULL, &to) > 0) {
-		while ((len = sanei_udp_recvfrom(fd, buf, 76, &ip)) == 76) {
+	while (select(fd + 1, &rfds, NULL, NULL, &to) > 0) {
+		if ((len = sanei_udp_recvfrom(fd, buf, 76, &ip)) == 76) {
 			DBG(5, " response from %s\n", ip);
 
 			/* minimal check, protocol unknown */
@@ -464,17 +474,29 @@ open_scanner(Epson_Scanner *s)
 
 			/* the scanner sends a kind of welcome msg */
 			read = e2_recv(s, buf, 5, &status);
-			if (read != 5)
+			if (read != 5) {
+				sanei_tcp_close(s->fd);
+				s->fd = -1;
 				return SANE_STATUS_IO_ERROR;
+			}
 
 			DBG(32, "welcome message received, locking the scanner...\n");
 
 			/* lock the scanner for use by sane */
-			sanei_epson_net_lock(s);
+			status = sanei_epson_net_lock(s);
+			if (status != SANE_STATUS_GOOD) {
+				DBG(1, "%s cannot lock scanner: %s\n", s->hw->sane.name,
+					sane_strstatus(status));
+
+				sanei_tcp_close(s->fd);
+				s->fd = -1;
+
+				return status;
+			}
 
 			DBG(32, "scanner locked\n");
 		}
-		
+
 	} else if (s->hw->connection == SANE_EPSON_SCSI)
 		status = sanei_scsi_open(s->hw->sane.name, &s->fd,
 					 sanei_epson2_scsi_sense_handler,
@@ -492,7 +514,7 @@ open_scanner(Epson_Scanner *s)
 		DBG(1, "disable any conflicting driver (like usblp).\n");
 	}
 
-	if (status != SANE_STATUS_GOOD) 
+	if (status != SANE_STATUS_GOOD)
 		DBG(1, "%s open failed: %s\n", s->hw->sane.name,
 			sane_strstatus(status));
 	else
@@ -564,12 +586,12 @@ static SANE_Status detect_scsi(struct Epson_Scanner *s)
 }
 
 static SANE_Status
-detect_usb(struct Epson_Scanner *s)
+detect_usb(struct Epson_Scanner *s, SANE_Bool assume_valid)
 {
 	SANE_Status status;
 	int vendor, product;
 	int i, numIds;
-	SANE_Bool is_valid;
+	SANE_Bool is_valid = assume_valid;
 
 	/* if the sanei_usb_get_vendor_product call is not supported,
 	 * then we just ignore this and rely on the user to config
@@ -581,7 +603,7 @@ detect_usb(struct Epson_Scanner *s)
 		DBG(1, "the device cannot be verified - will continue\n");
 	 	return SANE_STATUS_GOOD;
 	}
-	
+
 	/* check the vendor ID to see if we are dealing with an EPSON device */
 	if (vendor != SANE_EPSON_VENDOR_ID) {
 		/* this is not a supported vendor ID */
@@ -591,27 +613,28 @@ detect_usb(struct Epson_Scanner *s)
 	}
 
 	numIds = sanei_epson_getNumberOfUSBProductIds();
-	is_valid = SANE_FALSE;
 	i = 0;
 
 	/* check all known product IDs to verify that we know
 	   about the device */
-	while (i != numIds && !is_valid) {
-		if (product == sanei_epson_usb_product_ids[i])
+	while (i != numIds) {
+		if (product == sanei_epson_usb_product_ids[i]) {
 			is_valid = SANE_TRUE;
+			break;
+		}
 		i++;
 	}
 
 	if (is_valid == SANE_FALSE) {
-		DBG(1, "the device at %s is not a supported (product id=0x%x)\n",
+		DBG(1, "the device at %s is not supported (product id=0x%x)\n",
 			s->hw->sane.name, product);
 		return SANE_STATUS_INVAL;
 	}
-	
+
 	DBG(1, "found valid Epson scanner: 0x%x/0x%x (vendorID/productID)\n",
 		vendor, product);
 
-	return SANE_STATUS_GOOD;	    
+	return SANE_STATUS_GOOD;
 }
 
 static int num_devices;		/* number of scanners attached to backend */
@@ -637,7 +660,7 @@ scanner_create(struct Epson_Device *dev, SANE_Status *status)
 }
 
 static struct Epson_Scanner *
-device_detect(const char *name, int type, SANE_Status *status)
+device_detect(const char *name, int type, SANE_Bool assume_valid, SANE_Status *status)
 {
 	struct Epson_Scanner *s;
 	struct Epson_Device *dev;
@@ -660,7 +683,7 @@ device_detect(const char *name, int type, SANE_Status *status)
 		*status = SANE_STATUS_INVAL;
 		return NULL;
 	}
-	
+
 	/* alloc and clear our device structure */
 	dev = malloc(sizeof(*dev));
 	if (!dev) {
@@ -690,7 +713,7 @@ device_detect(const char *name, int type, SANE_Status *status)
 
 	} else if (dev->connection == SANE_EPSON_USB) {
 
-		*status = detect_usb(s);
+		*status = detect_usb(s, assume_valid);
 	}
 
 	if (*status != SANE_STATUS_GOOD)
@@ -736,7 +759,6 @@ device_detect(const char *name, int type, SANE_Status *status)
 
 close:
       	close_scanner(s);
-	free(s);
 	return NULL;
 }
 
@@ -749,12 +771,11 @@ attach(const char *name, int type)
 
 	DBG(7, "%s: devname = %s, type = %d\n", __func__, name, type);
 
-	s = device_detect(name, type, &status);
+	s = device_detect(name, type, 0, &status);
 	if(s == NULL)
 		return status;
 
       	close_scanner(s);
-	free(s);
 	return status;
 }
 
@@ -775,7 +796,7 @@ attach_one_usb(const char *dev)
 static SANE_Status
 attach_one_net(const char *dev)
 {
-        char name[39+4]; 
+        char name[39+4];
 
 	DBG(7, "%s: dev = %s\n", __func__, dev);
 
@@ -799,8 +820,9 @@ attach_one_config(SANEI_Config __sane_unused__ *config, const char *line)
 	int len = strlen(line);
 
 	DBG(7, "%s: len = %d, line = %s\n", __func__, len, line);
-	
+
 	if (sscanf(line, "usb %i %i", &vendor, &product) == 2) {
+
 		/* add the vendor and product IDs to the list of
 		   known devices before we call the attach function */
 
@@ -964,10 +986,10 @@ init_options(Epson_Scanner *s)
 	s->opt[OPT_MODE].size = max_string_size(mode_list);
 	s->opt[OPT_MODE].constraint_type = SANE_CONSTRAINT_STRING_LIST;
 	s->opt[OPT_MODE].constraint.string_list = mode_list;
-	s->val[OPT_MODE].w = 0;	/* Binary */
+	s->val[OPT_MODE].w = 0;	/* Lineart */
 
 	/* disable infrared on unsupported scanners */
-	if (!e2_model(s, "GT-X800") && !e2_model(s, "GT-X700") && !e2_model(s, "GT-X900"))
+	if (!e2_model(s, "GT-X800") && !e2_model(s, "GT-X700") && !e2_model(s, "GT-X900") && !e2_model(s, "GT-X980"))
 		mode_list[MODE_INFRARED] = NULL;
 
 	/* bit depth */
@@ -975,14 +997,13 @@ init_options(Epson_Scanner *s)
 	s->opt[OPT_BIT_DEPTH].title = SANE_TITLE_BIT_DEPTH;
 	s->opt[OPT_BIT_DEPTH].desc = SANE_DESC_BIT_DEPTH;
 	s->opt[OPT_BIT_DEPTH].type = SANE_TYPE_INT;
-	s->opt[OPT_BIT_DEPTH].unit = SANE_UNIT_NONE;
+	s->opt[OPT_BIT_DEPTH].unit = SANE_UNIT_BIT;
 	s->opt[OPT_BIT_DEPTH].constraint_type = SANE_CONSTRAINT_WORD_LIST;
 	s->opt[OPT_BIT_DEPTH].constraint.word_list = s->hw->depth_list;
-	s->opt[OPT_BIT_DEPTH].cap |= SANE_CAP_INACTIVE;
-	s->val[OPT_BIT_DEPTH].w = s->hw->depth_list[1];	/* the first "real" element is the default */
+	s->val[OPT_BIT_DEPTH].w = 8; /* default to 8 bit */
 
-	if (s->hw->depth_list[0] == 1)	/* only one element in the list -> hide the option */
-		s->opt[OPT_BIT_DEPTH].cap |= SANE_CAP_INACTIVE;
+	/* default is Lineart, disable depth selection */
+	s->opt[OPT_BIT_DEPTH].cap |= SANE_CAP_INACTIVE;
 
 	/* halftone */
 	s->opt[OPT_HALFTONE].name = SANE_NAME_HALFTONE;
@@ -1226,7 +1247,7 @@ init_options(Epson_Scanner *s)
 
 /*	if (!s->hw->cmd->set_color_correction)
 		s->opt[OPT_FILM_TYPE].cap |= SANE_CAP_INACTIVE;
-*/	
+*/
 
 	/* mirror */
 	s->opt[OPT_MIRROR].name = "mirror";
@@ -1444,6 +1465,8 @@ sane_open(SANE_String_Const name, SANE_Handle *handle)
 
 	DBG(7, "%s: name = %s\n", __func__, name);
 
+	*handle = NULL;
+
 	/* probe if empty device name provided */
 	if (l == 0) {
 
@@ -1455,7 +1478,7 @@ sane_open(SANE_String_Const name, SANE_Handle *handle)
 		}
 
 		s = device_detect(first_dev->sane.name, first_dev->connection,
-					&status);
+					0, &status);
 		if (s == NULL) {
 			DBG(1, "cannot open a perfectly valid device (%s),"
 				" please report to the authors\n", name);
@@ -1465,33 +1488,33 @@ sane_open(SANE_String_Const name, SANE_Handle *handle)
 	} else {
 
 		if (strncmp(name, "net:", 4) == 0) {
-			s = device_detect(name, SANE_EPSON_NET, &status);
+			s = device_detect(name, SANE_EPSON_NET, 0, &status);
 			if (s == NULL)
 				return status;
 		} else if (strncmp(name, "libusb:", 7) == 0) {
-			s = device_detect(name, SANE_EPSON_USB, &status);
+			s = device_detect(name, SANE_EPSON_USB, 1, &status);
 			if (s == NULL)
 				return status;
 		} else if (strncmp(name, "pio:", 4) == 0) {
-			s = device_detect(name, SANE_EPSON_PIO, &status);
+			s = device_detect(name, SANE_EPSON_PIO, 0, &status);
 			if (s == NULL)
 				return status;
 		} else {
-		
+
 			/* as a last resort, check for a match
 			 * in the device list. This should handle SCSI
 			 * devices and platforms without libusb.
 			 */
-			
+
 			if (first_dev == NULL)
 				probe_devices();
 
-			s = device_detect(name, SANE_EPSON_NODEV, &status);
+			s = device_detect(name, SANE_EPSON_NODEV, 0, &status);
 			if (s == NULL) {
 				DBG(1, "invalid device name: %s\n", name);
 				return SANE_STATUS_INVAL;
 			}
-		}		
+		}
 	}
 
 
@@ -1501,8 +1524,6 @@ sane_open(SANE_String_Const name, SANE_Handle *handle)
 
 	init_options(s);
 
-	*handle = (SANE_Handle) s;
-
 	status = open_scanner(s);
 	if (status != SANE_STATUS_GOOD) {
 		free(s);
@@ -1510,18 +1531,23 @@ sane_open(SANE_String_Const name, SANE_Handle *handle)
 	}
 
 	status = esci_reset(s);
-	if (status != SANE_STATUS_GOOD)
+	if (status != SANE_STATUS_GOOD) {
 		close_scanner(s);
-	
-	return status;
+		return status;
+	}
+
+	*handle = (SANE_Handle)s;
+
+	return SANE_STATUS_GOOD;
 }
 
 void
 sane_close(SANE_Handle handle)
 {
-	int i;
 	Epson_Scanner *s;
 
+	DBG(1, "* %s\n", __func__);
+
 	/*
 	 * XXX Test if there is still data pending from
 	 * the scanner. If so, then do a cancel
@@ -1529,15 +1555,7 @@ sane_close(SANE_Handle handle)
 
 	s = (Epson_Scanner *) handle;
 
-	if (s->fd != -1)
-		close_scanner(s);
-
-	for (i = 0; i < LINES_SHUFFLE_MAX; i++) {
-		if (s->line_buffer[i] != NULL)
-			free(s->line_buffer[i]);
-	}
-
-	free(s);
+	close_scanner(s);
 }
 
 const SANE_Option_Descriptor *
@@ -1740,8 +1758,9 @@ change_source(Epson_Scanner *s, SANE_Int optindex, char *value)
 			s->val[OPT_ADF_MODE].w = 0;
 		}
 
-		DBG(1, "adf activated (%d %d)\n", s->hw->use_extension,
-		    s->hw->duplex);
+		DBG(1, "adf activated (ext: %d, duplex: %d)\n",
+			s->hw->use_extension,
+			s->hw->duplex);
 
 	} else if (strcmp(TPU_STR, value) == 0 || strcmp(TPU_STR2, value) == 0) {
 	        if (strcmp(TPU_STR, value) == 0) {
@@ -1895,6 +1914,8 @@ setvalue(SANE_Handle handle, SANE_Int option, void *value, SANE_Int *info)
 
 		sval->w = optindex;
 
+		DBG(17, "%s: setting mode to %d\n", __func__, optindex);
+
 		/* halftoning available only on bw scans */
 		if (s->hw->cmd->set_halftoning != 0)
 			setOptionState(s, mode_params[optindex].depth == 1,
@@ -1909,16 +1930,18 @@ setvalue(SANE_Handle handle, SANE_Int option, void *value, SANE_Int *info)
 
 		/* if binary, then disable the bit depth selection */
 		if (optindex == 0) {
+			DBG(17, "%s: disabling bit depth selection\n", __func__);
 			s->opt[OPT_BIT_DEPTH].cap |= SANE_CAP_INACTIVE;
 		} else {
-			if (s->hw->depth_list[0] == 1)
-				s->opt[OPT_BIT_DEPTH].cap |=
-					SANE_CAP_INACTIVE;
-			else {
-				s->opt[OPT_BIT_DEPTH].cap &=
-					~SANE_CAP_INACTIVE;
-				s->val[OPT_BIT_DEPTH].w =
-					mode_params[optindex].depth;
+			if (s->hw->depth_list[0] == 1) {
+				DBG(17, "%s: only one depth is available\n", __func__);
+				s->opt[OPT_BIT_DEPTH].cap |= SANE_CAP_INACTIVE;
+			} else {
+
+				DBG(17, "%s: enabling bit depth selection\n", __func__);
+
+				s->opt[OPT_BIT_DEPTH].cap &= ~SANE_CAP_INACTIVE;
+				s->val[OPT_BIT_DEPTH].w = mode_params[optindex].depth;
 			}
 		}
 
@@ -2063,7 +2086,10 @@ sane_start(SANE_Handle handle)
 	Epson_Device *dev = s->hw;
 	SANE_Status status;
 
-	DBG(5, "%s\n", __func__);
+	DBG(5, "* %s\n", __func__);
+
+	s->eof = SANE_FALSE;
+	s->canceling = SANE_FALSE;
 
 	/* check if we just have finished working with the ADF */
 	status = e2_check_adf(s);
@@ -2105,7 +2131,7 @@ sane_start(SANE_Handle handle)
 			return status;
 	}
 
-	
+
 	if (s->val[OPT_COLOR_CORRECTION].w == CORR_AUTO) { /* Automatic */
 
 		DBG(1, "using built in CCT profile\n");
@@ -2115,7 +2141,7 @@ sane_start(SANE_Handle handle)
 
 
 		if (0) { /* XXX TPU */
- 
+
 		        /* XXX check this */
 			if (s->val[OPT_FILM_TYPE].w == 0)
 				e2_load_cct_profile(s, CCTP_COLORPOS);
@@ -2126,7 +2152,7 @@ sane_start(SANE_Handle handle)
 			e2_load_cct_profile(s, CCTP_REFLECTIVE);
 		}
 	}
-                                                    
+
 	/* ESC m, user defined color correction */
 	if (s->hw->cmd->set_color_correction_coefficients
 		&& correction_userdefined[s->val[OPT_COLOR_CORRECTION].w]) {
@@ -2188,9 +2214,7 @@ sane_start(SANE_Handle handle)
 	if (s->buf == NULL)
 		return SANE_STATUS_NO_MEM;
 
-	s->eof = SANE_FALSE;
 	s->ptr = s->end = s->buf;
-	s->canceling = SANE_FALSE;
 
 	/* feed the first sheet in the ADF */
 	if (dev->ADF && dev->use_extension && dev->cmd->feed) {
@@ -2261,8 +2285,14 @@ sane_read(SANE_Handle handle, SANE_Byte *data, SANE_Int max_length,
 	SANE_Status status;
 	Epson_Scanner *s = (Epson_Scanner *) handle;
 
-	if (s->buf == NULL || s->canceling)
-		return SANE_STATUS_CANCELLED;
+	DBG(18, "* %s: eof: %d, canceling: %d\n",
+		__func__, s->eof, s->canceling);
+
+	/* sane_read called before sane_start? */
+	if (s->buf == NULL) {
+		DBG(1, "%s: buffer is NULL", __func__);
+		return SANE_STATUS_INVAL;
+	}
 
 	*length = 0;
 
@@ -2271,9 +2301,12 @@ sane_read(SANE_Handle handle, SANE_Byte *data, SANE_Int max_length,
 	else
 		status = e2_block_read(s);
 
-	if (status == SANE_STATUS_CANCELLED) {
+	/* The scanning operation might be canceled by the scanner itself
+	 * or the fronted program
+	 */
+	if (status == SANE_STATUS_CANCELLED || s->canceling) {
 		e2_scan_finish(s);
-		return status;
+		return SANE_STATUS_CANCELLED;
 	}
 
 	/* XXX if FS G and STATUS_IOERR, use e2_check_extended_status */
@@ -2284,9 +2317,9 @@ sane_read(SANE_Handle handle, SANE_Byte *data, SANE_Int max_length,
 
 	e2_copy_image_data(s, data, max_length, length);
 
-	DBG(18, "%d lines read, eof: %d, status: %d\n",
+	DBG(18, "%d lines read, eof: %d, canceling: %d, status: %d\n",
 		*length / s->params.bytes_per_line,
-		s->eof, status);
+		s->canceling, s->eof, status);
 
 	/* continue reading if appropriate */
 	if (status == SANE_STATUS_GOOD)
@@ -2309,6 +2342,8 @@ sane_cancel(SANE_Handle handle)
 {
 	Epson_Scanner *s = (Epson_Scanner *) handle;
 
+	DBG(1, "* %s\n", __func__);
+
 	s->canceling = SANE_TRUE;
 }
 
