@@ -41,6 +41,12 @@
 #include "sane/config.h"
 
 #include <ctype.h>
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
 #include <unistd.h>
 
 #include "sane/saneopts.h"
