@@ -161,7 +161,6 @@ SANE_Status
 sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
 {
   char line[PATH_MAX];		/*Line from config file */
-  int len;			/*Length of string from config file */
   FILE *config_file;		/*Handle to config file of this backend */
 
   DBG_INIT ();
@@ -189,7 +188,6 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
     {
       if ((line[0] == '#') || (line[0] == '\0'))	/*comment line or empty line */
 	continue;
-      len = strlen (line);	/*sanei_config_read guaranty, it's not more then PATH_MAX-1 */
       strcpy (scanner_path, line);	/*so, we choose last in file (uncommented) */
     }
 
@@ -334,7 +332,7 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
       return SANE_STATUS_IO_ERROR;
     }
   DBG (1, "sane_open: Device found.All are green.");
-  *handle = (SANE_Handle) scanner_d;
+  *handle = (SANE_Handle) (unsigned long)scanner_d;
 
   return SANE_STATUS_GOOD;
 }
@@ -344,7 +342,7 @@ sane_close (SANE_Handle handle)
 {
   DBG (2, "sane_close\n");
   /*We support only single device - so ignore handle (FIX IT LATER) */
-  if ((handle != (SANE_Handle) scanner_d) || (scanner_d == -1))
+  if ((handle != (SANE_Handle) (unsigned long)scanner_d) || (scanner_d == -1))
     return;			/* wrong device */
   StandByScanner ();
   CloseScanner (scanner_d);
@@ -355,7 +353,7 @@ const SANE_Option_Descriptor *
 sane_get_option_descriptor (SANE_Handle handle, SANE_Int option)
 {
   DBG (2, "sane_get_option_descriptor: option = %d\n", option);
-  if ((handle != (SANE_Handle) scanner_d) || (scanner_d == -1))
+  if ((handle != (SANE_Handle) (unsigned long)scanner_d) || (scanner_d == -1))
     return NULL;		/* wrong device */
 
   if (option < 0 || option >= NELEMS (sod))	/*No real options supported */
@@ -368,7 +366,7 @@ SANE_Status
 sane_control_option (SANE_Handle handle, SANE_Int option,
 		     SANE_Action action, void *value, SANE_Int * info)
 {
-  if ((handle != (SANE_Handle) scanner_d) || (scanner_d == -1))
+  if ((handle != (SANE_Handle) (unsigned long)scanner_d) || (scanner_d == -1))
     return SANE_STATUS_INVAL;	/* wrong device */
 
   if ((option >= NELEMS (sod)) || (option < 0))	/*Supported only this option */
@@ -421,7 +419,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
 {
   DBG (2, "sane_get_parameters\n");
 
-  if ((handle != (SANE_Handle) scanner_d) || (scanner_d == -1))
+  if ((handle != (SANE_Handle) (unsigned long)scanner_d) || (scanner_d == -1))
     return SANE_STATUS_INVAL;	/* wrong device */
 
   /*Ignore handle parameter for now. FIX it latter. */
@@ -442,7 +440,7 @@ sane_start (SANE_Handle handle)
   int i;
   DBG (2, "sane_start\n");
 
-  if ((handle != (SANE_Handle) scanner_d) || (scanner_d == -1))
+  if ((handle != (SANE_Handle) (unsigned long)scanner_d) || (scanner_d == -1))
     return SANE_STATUS_IO_ERROR;
 
   CallFunctionWithParameter (0x93, 2);
@@ -507,7 +505,7 @@ sane_read (SANE_Handle handle, SANE_Byte * data,
       return SANE_STATUS_INVAL;
     }
 
-  if ((handle != (SANE_Handle) scanner_d) || (scanner_d == -1))
+  if ((handle != (SANE_Handle) (unsigned long)scanner_d) || (scanner_d == -1))
     {
       DBG (1, "sane_read: unknown handle\n");
       return SANE_STATUS_INVAL;
@@ -1067,7 +1065,7 @@ CalibrateScanElements ()
   SANE_Byte CurrentLowTransferBorder;
   SANE_Byte CurrentUpSaveBorder;
   SANE_Byte CurrentLowSaveBorder;
-  SANE_Byte CurrentSpeed1, CurrentSpeed2; 
+  SANE_Byte CurrentSpeed1, CurrentSpeed2;
   SANE_Byte CorrectionValue;
   SANE_Byte FilteredBuffer[2570];
 
@@ -1169,14 +1167,14 @@ CalibrateScanElements ()
       CallFunctionWithParameter (0x91, 0);	/*Stop engine. */
 
      /*Note: if first read failed, junk would be calculated, but if previous
-	read was succeded, but last one failed, previous data'ld be used.
+	read was succeeded, but last one failed, previous data'ld be used.
      */
      for(Temp = 0, j = 0; j < 2570; j++)
      Temp += FilteredBuffer[j];
      Temp /= 2570;
 
      if((Average == 0)||(Average > Temp))
-     Average = Temp; 
+     Average = Temp;
     }
 
     for(Index = 0; Index < 3; Index++) /*Three color components*/
@@ -1192,7 +1190,7 @@ CalibrateScanElements ()
 	CallFunctionWithParameter (0x92, CurrentLowTransferBorder|0x80);
 	for(j=2999; j>0; j--)
 	    CallFunctionWithParameter (0xC6, 0);
-	
+
 	CurrentUpSaveBorder = arUpSaveBorders[Index];
 	CallFunctionWithParameter (CurrentUpSaveBorder, 0xFF);
 
@@ -1201,9 +1199,9 @@ CalibrateScanElements ()
 	CallFunctionWithParameter (0x90,0);
 	Calibration = 0x80;
 	CallFunctionWithParameter (CurrentUpSaveBorder, 0x80);
-	
+
 	CurrentSpeed1 = CurrentSpeed2 = arSpeeds[Index];
-	
+
 	for(CorrectionValue = 0x40; CorrectionValue != 0;CorrectionValue >>= 2)
 	{
 	    CallFunctionWithParameter (0x91, CurrentSpeed2);
@@ -1237,11 +1235,11 @@ CalibrateScanElements ()
 		Perhaps, we can optimize it in future.*/
 	    WriteScannerRegister (0x91, 0);
 	    usleep(10);
-	    
+
 	    for(Temp = 0,j = 0; j < 16;j++)
 		Temp += FilteredBuffer[509+j]; /*At this offset calcalates HP's driver.*/
 	    Temp /= 16;
-	    
+
 	    if(Average > Temp)
 	    {
 		Calibration += CorrectionValue;
@@ -1249,7 +1247,7 @@ CalibrateScanElements ()
 	    }
 	    else
 		Calibration -= CorrectionValue;
-	    
+
 	    WriteScannerRegister (CurrentUpSaveBorder, Calibration);
 	}/*By CorrectionValue we tune UpSaveBorder*/
 
