@@ -1,8 +1,8 @@
 /* SANE - Scanner Access Now Easy.
 
+   Copyright (C) 2011-2019 Rolf Bensch <rolf at bensch hyphen online dot de>
    Copyright (C) 2007-2008 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
    Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
-   Copyright (C) 2011-2012 Rolf Bensch <rolf at bensch hyphen online dot de>
 
    This file is part of the SANE package.
 
@@ -104,16 +104,19 @@ typedef uint16_t uint16_t;
 typedef uint32_t uint32_t;
 #endif /* HAVE_STDINT_H */
 
-#ifdef HAVE_INTTYPES_H 
+#ifdef HAVE_INTTYPES_H
 # include <inttypes.h>          /* available in ISO C99 */
 #endif /* HAVE_INTTYPES_H */
 
 /** \addtogroup API
  *  @{ */
+/** Don't forget to update the backend version in the SANE Backend specification
+ *  file: doc/descriptions/pixma.desc !!!
+ */
 /** \name Version of the driver */
 /**@{*/
 #define PIXMA_VERSION_MAJOR 0
-#define PIXMA_VERSION_MINOR 17
+#define PIXMA_VERSION_MINOR 23
 #define PIXMA_VERSION_BUILD 0
 /**@}*/
 
@@ -149,15 +152,26 @@ typedef uint32_t uint32_t;
 #define PIXMA_CAP_CCD          (1 << 8)
 #define PIXMA_CAP_LINEART      (1 << 9)
 #define PIXMA_CAP_NEGATIVE     (1 << 10)
+#define PIXMA_CAP_TPUIR        ((1 << 11) | PIXMA_CAP_TPU)
+#define PIXMA_CAP_ADF_WAIT     (1 << 12)
+#define PIXMA_CAP_ADF_JPEG     (1 << 13)
 #define PIXMA_CAP_EXPERIMENT   (1 << 31)
 /**@}*/
 
-/** \name Button events returned by pixma_wait_event() */
+/** \name Button events and related information returned by pixma_wait_event() */
 /**@{*/
-#define PIXMA_EV_NONE        0
-#define PIXMA_EV_ACTION_MASK (0xff)
-#define PIXMA_EV_BUTTON1     (1 << 8)
-#define PIXMA_EV_BUTTON2     (2 << 8)
+#define PIXMA_EV_NONE          0
+#define PIXMA_EV_ACTION_MASK   (0xffffff)
+#define PIXMA_EV_BUTTON1       (1 << 24)
+#define PIXMA_EV_BUTTON2       (2 << 24)
+#define PIXMA_EV_TARGET_MASK   (0xff)
+#define PIXMA_EV_ORIGINAL_MASK (0xff00)
+#define PIXMA_EV_DPI_MASK      (0xff0000)
+
+#define GET_EV_TARGET(x) (x & PIXMA_EV_TARGET_MASK)
+#define GET_EV_ORIGINAL(x) ( (x & PIXMA_EV_ORIGINAL_MASK) >> 8 )
+#define GET_EV_DPI(x) ( (x & PIXMA_EV_DPI_MASK) >> 16 )
+
 /**@}*/
 /** @} end of API group */
 
@@ -213,7 +227,9 @@ typedef enum pixma_scan_mode_t
   PIXMA_SCAN_MODE_COLOR_48,
   PIXMA_SCAN_MODE_GRAY_16,
   /* 1 bit lineart scan mode */
-  PIXMA_SCAN_MODE_LINEART
+  PIXMA_SCAN_MODE_LINEART,
+  /* TPUIR scan mode */
+  PIXMA_SCAN_MODE_TPUIR
 } pixma_scan_mode_t;
 
 typedef enum pixma_hardware_status_t
@@ -263,7 +279,7 @@ struct pixma_scan_param_t
      *  line_size >= depth / 8 * channels * w <br>
      *  This field will be set by pixma_check_scan_param(). */
   uint64_t line_size;
-  
+
     /** Size in bytes of the whole image.
      *  image_size = line_size * h <br>
      *  This field will be set by pixma_check_scan_param(). */
@@ -295,11 +311,14 @@ struct pixma_scan_param_t
   unsigned x, y, w, h,   xs, wx;
   /*@} */
 
-  /** Flag indicating whether the offset correction for TPU scans 
+  /** Flag indicating whether the offset correction for TPU scans
    *  was already performed (to avoid repeated corrections).
    *  Currently only used in pixma_mp810.c sub-driver */
   unsigned tpu_offset_added;
 
+  /* Flag indicating if data from scanner will be in JPEG format */
+  unsigned mode_jpeg;
+
   /** Flag indicating whether a software-lineart scan is in progress
    *  0 = other scan
    *  1 = software-lineart scan */
@@ -326,6 +345,10 @@ struct pixma_scan_param_t
 
     /** The current page # in the same ADF scan session, 0 in non ADF */
   unsigned adf_pageid;
+
+  /** adf-wait */
+  unsigned adf_wait;
+  unsigned frontend_cancel;
 };
 
 /** PIXMA model information */
@@ -341,10 +364,14 @@ struct pixma_config_t
   const pixma_scan_ops_t *ops;	  /**< Subdriver ops */
   unsigned xdpi;	     /**< Maximum horizontal resolution[DPI] */
   unsigned ydpi;	     /**< Maximum vertical resolution[DPI] */
-  unsigned ext_min_dpi;    /**< Maximum horizontal resolution[DPI] for tpu/adf
-                             *  only needed if ADF/TPU has another min. dpi value than 75 dpi */
-  unsigned ext_max_dpi;    /**< Maximum vertical resolution[DPI] for tpu/adf
-                             *  only needed if ADF/TPU has another max. dpi value than xdpi */
+  unsigned adftpu_min_dpi;    /**< Maximum horizontal resolution[DPI] for adf/tpu
+                                 *  only needed if ADF/TPU has another min. dpi value than 75 dpi */
+  unsigned adftpu_max_dpi;    /**< Maximum vertical resolution[DPI] for adf/tpu
+                                 *  only needed if ADF/TPU has another max. dpi value than xdpi */
+  unsigned tpuir_min_dpi;       /**< Minimum resolution[DPI] for tpu-ir
+                                   *  only needed if TPU-IR has another min. dpi value than 75 dpi */
+  unsigned tpuir_max_dpi;       /**< Maximum resolution[DPI] for tpu-ir
+                                   *  only needed if TPU-IR has another max. dpi value than xdpi */
   unsigned width;	     /**< Maximum width of scannable area in pixels at 75DPI */
   unsigned height;	   /**< Maximum height of scannable area in pixels at 75DPI */
   unsigned cap;		     /**< Capability bitfield \see PIXMA_CAP_* */
