@@ -20,7 +20,7 @@
 
    This file implements a SANE backend for Siemens 9036 flatbed scanners.  */
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -31,27 +31,27 @@
 # define PATH_MAX	1024
 #endif
 
-#include "sane/sane.h"
-#include "sane/sanei.h"
-#include "sane/sanei_config.h"
-#include "sane/saneopts.h"
-#include "sane/sanei_scsi.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei.h"
+#include "../include/sane/sanei_config.h"
+#include "../include/sane/saneopts.h"
+#include "../include/sane/sanei_scsi.h"
 #include "s9036.h"
 
 #define BACKEND_NAME	s9036
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
-#define MM_PER_INCH	25.4
 
 #undef Byte
 #define Byte SANE_Byte
 
+static const SANE_Device **devlist = NULL;
 static int num_devices;
 static S9036_Device *s9036_devices;
 
 
 /* sets loc_s bytes long value at offset loc in scsi command to value size  */
-static void 
+static void
 set_size (Byte * loc, int loc_s, size_t size)
 {
   int i;
@@ -62,7 +62,7 @@ set_size (Byte * loc, int loc_s, size_t size)
     }
 }
 
-static long 
+static long
 reserve_unit (int fd)
 {
   const Byte scsi_reserve[] =
@@ -73,7 +73,7 @@ reserve_unit (int fd)
   return sanei_scsi_cmd (fd, scsi_reserve, sizeof (scsi_reserve), 0, 0);
 }
 
-static long 
+static long
 release_unit (int fd)
 {
   const Byte scsi_release[] =
@@ -654,9 +654,9 @@ init_options (S9036_Scanner * s)
 
   static const SANE_Range percentage_range =
   {
-    -100 << SANE_FIXED_SCALE_SHIFT,	/* minimum */
-    100 << SANE_FIXED_SCALE_SHIFT,	/* maximum */
-    1 << SANE_FIXED_SCALE_SHIFT	/* quantization */
+    SANE_FIX(-100),	/* minimum */
+    SANE_FIX(100),	/* maximum */
+    SANE_FIX(1)         /* quantization */
   };
 
   static const SANE_Range automatic_adjust_range =
@@ -871,15 +871,17 @@ sane_exit (void)
 	sane_close (dev->handle);
       free (dev);
     }
+
+  if (devlist)
+    free (devlist);
 }
 
 SANE_Status
 sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
 {
-  static const SANE_Device **devlist = 0;
   S9036_Device *dev;
   int i;
-  
+
   local_only = local_only; /* silence compilation warnings */
 
   if (devlist)
@@ -1235,7 +1237,7 @@ sane_start (SANE_Handle handle)
   return SANE_STATUS_GOOD;
 }
 
-static void 
+static void
 copy_buffer (S9036_Scanner * s, SANE_Byte ** buf, SANE_Int * max_len,
 	     SANE_Int * len)
 {
@@ -1325,7 +1327,7 @@ SANE_Status
 sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 {
   handle = handle; /* silence compilation warnings */
-  
+
   DBG (1, "sane_set_io_mode(%d)\n", non_blocking);
 
   return (non_blocking == SANE_TRUE) ?
