@@ -1,5 +1,5 @@
 /* sane - Scanner Access Now Easy.
-   Copyright (C) 2001-2008 Stphane Voltz <stef.dev@free.fr>
+   Copyright (C) 2001-2012 Stéphane Voltz <stef.dev@free.fr>
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -41,10 +41,10 @@
    This file implements a SANE backend for Umax PP flatbed scanners.  */
 
 /* CREDITS:
-   Started by being a mere copy of mustek_pp 
-   by Jochen Eisinger <jochen.eisinger@gmx.net> 
-   then evolved in its own thing                                       
-   
+   Started by being a mere copy of mustek_pp
+   by Jochen Eisinger <jochen.eisinger@gmx.net>
+   then evolved in its own thing
+
    support for the 610P has been made possible thank to an hardware donation
    from William Stuart
    */
@@ -107,8 +107,8 @@
  *  see Changelog
  */
 
-#define UMAX_PP_BUILD   700
-#define UMAX_PP_STATE   "testing"
+#define UMAX_PP_BUILD   2301
+#define UMAX_PP_STATE   "release"
 
 static int num_devices = 0;
 static Umax_PP_Descriptor *devlist = NULL;
@@ -388,7 +388,7 @@ umax_pp_try_ports (SANEI_Config * config, char **ports)
                      ports[i]);
               else
                 DBG (3,
-                     "umax_pp_try_ports: attach to port `%s' successfull\n",
+                     "umax_pp_try_ports: attach to port `%s' successful\n",
                      ports[i]);
             }
           free (ports[i]);
@@ -1001,9 +1001,9 @@ sane_exit (void)
   for (i = 0; i < num_devices; i++)
     {
       free (devlist[i].port);
-      free (devlist[i].sane.name);
-      free (devlist[i].sane.model);
-      free (devlist[i].sane.vendor);
+      free ((void *) devlist[i].sane.name);
+      free ((void *) devlist[i].sane.model);
+      free ((void *) devlist[i].sane.vendor);
     }
 
   if (devlist != NULL)
